#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"


#define RQ_GET_LDR1    1
#define RQ_GET_LDR2    2
#define RQ_GET_LDR3    3

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

uint16_t read_adc(uint8_t channel)
{
    ADMUX = 0b01000000; // เลือกใช้แรงดันอ้างอิงจากขา AREF
    ADMUX |= channel;   // ตั้ง MUX ให้ชี้ไปยัง channel
    ADCSRA = 0b11000110;
    //         ||   -+-
    //         ||    |
    //         ||    +-> ปรับคล็อกของ ADC ให้เป็น 1/64 ของคล็อกระบบ
    //         |+->เริ่มแปลงค่าแอนะล็อกเป็นดิจิทัล (บิต ADSC)
    //         |
    //         +-> Enable ADC (บิต ADEN)

    while ((ADCSRA & (1<<ADSC))) // รอจนบิต ADSC (บิต 6) กลายเป็น 0
        ;
    return ADCL + ADCH*256;  // ผลลัพธ์ถูกเก็บอยู่ในรีจีสเตอร์ ADCL และ ADCH
}

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t switch_state;  /* must stay valid when usbFunctionSetup returns */
    static uint16_t result,light1,light2,light3;

    if (rq->bRequest == RQ_GET_LDR1){
		
		
		result = read_adc(PC0);
		light1 = result & 0b0000001111111111;
		
		usbMsgPtr = (uint16_t*) &light1;
		return sizeof(light1);

	}

	else if (rq->bRequest == RQ_GET_LDR2){
		
		
		result = read_adc(PC2);
		light2 = result & 0b0000001111111111;
		
		usbMsgPtr = (uint16_t*) &light2;
		return sizeof(light2);

	}

	else if (rq->bRequest == RQ_GET_LDR3){
		
		
		result = read_adc(PC4);
		light3 = result & 0b0000001111111111;
		
		usbMsgPtr = (uint16_t*) &light3;
		return sizeof(light3);

	}

    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{
    DDRC  &= ~((1<<PC0)|(1<<PC2)|(1<<PC4)); // PC0..PC2 = Output, PC3..PC5 = Input
    

    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        usbPoll();
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
